"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var core_1 = require("@angular/core");
var renderer_1 = require("../../renderer/renderer");
var wrapper_1 = require("../../wrapper/wrapper");
var component_1 = require("./component");
var ANDROID_INPUTS = ['enabled', 'mode', 'prompt'];
var IOS_INPUTS = ['itemStyle'];
var ANDROID_BINDINGS = "[enabled]=\"_enabled\" [mode]=\"_mode\" [prompt]=\"_prompt\" (topSelect)=\"_handleSelect($event)\" [selected]=\"_selectedValue\"";
var IOS_BINDINGS = "[itemStyle]=\"_itemStyle\" onChange=\"true\" (topChange)=\"_handleSelect($event)\" [selectedIndex]=\"_selectedValue\"";
/**
 * A component for displaying a picker.
 *
 * ```
@Component({
  selector: 'sample',
  template: `<Picker [selectedValue]="selected" prompt="Please select an item" [items]="items"
    [style]="{width: 80}" (select)="selected=$event"></Picker>`
})
export class Sample {
  selected: number = 0;
  items: Array<any> = [{label: 'aaa', value: 'a'}, {label: 'bbb', value: 'b'}, {label: 'ccc', value: 'c'},
    {label: 'ddd', value: 'd'}, {label: 'eee', value: 'e'}];
}
 * ```
 * @style https://facebook.github.io/react-native/docs/view.html#style
 */
var Picker = (function (_super) {
    __extends(Picker, _super);
    function Picker(wrapper) {
        _super.call(this, wrapper);
        //Events
        /**
         * To be documented
         */
        this.select = new core_1.EventEmitter();
        //Properties
        this._items = [];
        this._mode = 'dialog';
        this._itemStyle = false;
        this.setDefaultStyle({ height: wrapper.isAndroid() ? 50 : 216 });
    }
    Object.defineProperty(Picker.prototype, "selectedValue", {
        /**
         * To be documented
         */
        set: function (value) { this._selectedValue = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "items", {
        /**
         * To be documented
         */
        set: function (value) {
            var _this = this;
            this._items = value.map(function (item) {
                var res = { label: item.label, value: item.value };
                if (item.color) {
                    res['color'] = _this.processColor(item.color);
                }
                return res;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "enabled", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._enabled = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "mode", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._mode = this.processEnum(value, ['dialog', 'dropdown']); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "prompt", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._prompt = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "itemStyle", {
        /**
         * To be documented
         * @platform ios
         */
        set: function (value) { this._itemStyle = value; },
        enumerable: true,
        configurable: true
    });
    Picker.prototype._handleSelect = function (event) {
        //Event example: {position: 3}
        this._selectedValue = this._wrapper.isAndroid() ? event.position : event.newIndex;
        this.select.emit(this._selectedValue);
    };
    __decorate([
        core_1.Output(), 
        __metadata('design:type', core_1.EventEmitter)
    ], Picker.prototype, "select", void 0);
    Picker = __decorate([
        core_1.Component({
            selector: 'Picker',
            inputs: [
                'items', 'selectedValue'
            ].concat(component_1.GENERIC_INPUTS).concat(wrapper_1.isAndroid() ? ANDROID_INPUTS : IOS_INPUTS),
            template: "\n  <native-dialogpicker *ngIf=\"_mode == 'dialog'\" [items]=\"_items\"\n  " + component_1.GENERIC_BINDINGS + " " + (wrapper_1.isAndroid() ? ANDROID_BINDINGS : IOS_BINDINGS) + "></native-dialogpicker>\n  <native-dropdownpicker *ngIf=\"_mode == 'dropdown'\" [items]=\"_items\"\n  " + component_1.GENERIC_BINDINGS + " " + (wrapper_1.isAndroid() ? ANDROID_BINDINGS : IOS_BINDINGS) + "></native-dropdownpicker>"
        }),
        __param(0, core_1.Inject(renderer_1.REACT_NATIVE_WRAPPER)), 
        __metadata('design:paramtypes', [wrapper_1.ReactNativeWrapper])
    ], Picker);
    return Picker;
}(component_1.HighLevelComponent));
exports.Picker = Picker;
