var ANDROID_INPUTS = ['collapsable', 'accessibilityLiveRegion', 'accessibilityComponentType',
    'importantForAccessibility', 'needsOffscreenAlphaCompositing', 'renderToHardwareTextureAndroid ', 'nativeBackgroundAndroid'];
var IOS_INPUTS = ['accessibilityTraits', 'shouldRasterizeIOS'];
export var GENERIC_INPUTS = [
    //Both platforms
    'accessible', 'accessibilityLabel', 'testID', 'pointerEvents', 'removeClippedSubviews', 'onLayout',
    //Style
    'styleSheet', 'style', 'accessibilityTraits', 'shouldRasterizeIOS'
];
var ANDROID_BINDINGS = "[collapsable]=\"_collapsable\" [accessibilityLiveRegion]=\"_accessibilityLiveRegion\"\n  [accessibilityComponentType]=\"_accessibilityComponentType\" [importantForAccessibility]=\"_importantForAccessibility\"\n  [needsOffscreenAlphaCompositing]=\"_needsOffscreenAlphaCompositing\" [renderToHardwareTextureAndroid]=\"_renderToHardwareTextureAndroid\"\n  [nativeBackgroundAndroid]=\"_nativeBackgroundAndroid\"";
var IOS_BINDINGS = "[accessibilityTraits]=\"_accessibilityTraits\" [shouldRasterizeIOS]=\"_shouldRasterizeIOS\"";
export var GENERIC_BINDINGS = "[accessible]=\"_accessible\" [accessibilityLabel]=\"_accessibilityLabel\" [testID]=\"_testID\"\n  [pointerEvents]=\"_pointerEvents\" [removeClippedSubviews]=\"_removeClippedSubviews\" [_onLayout]=\"_onLayout\" [styleSheet]=\"_styleSheet\"\n  [style]=\"_style\" [accessibilityTraits]=\"_accessibilityTraits\" [shouldRasterizeIOS]=\"_shouldRasterizeIOS\"";
/**
 * An abstract component which defines inputs available in all components.
 */
var HighLevelComponent = (function () {
    function HighLevelComponent(_wrapper) {
        //Style
        this._defaultStyle = {};
        this._wrapper = _wrapper;
    }
    Object.defineProperty(HighLevelComponent.prototype, "styleSheet", {
        /**
         * To be documented
         */
        set: function (value) { this._styleSheet = [this._defaultStyle].concat(Array.isArray(value) ? value : [value]); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "style", {
        /**
         * To be documented
         */
        set: function (value) { this._style = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "accessible", {
        /**
         * To be documented
         */
        set: function (value) { this._accessible = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "accessibilityLabel", {
        /**
         * To be documented
         */
        set: function (value) { this._accessibilityLabel = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "testID", {
        /**
         * To be documented
         */
        set: function (value) { this._testID = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "pointerEvents", {
        /**
         * To be documented
         */
        set: function (value) { this._pointerEvents = this.processEnum(value, ['auto', 'box-none', 'none', 'box-only']); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "removeClippedSubviews", {
        /**
         * To be documented
         */
        set: function (value) { this._removeClippedSubviews = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "onLayout", {
        /**
         * To be documented
         */
        set: function (value) { this._onLayout = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "collapsable", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._collapsable = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "accessibilityLiveRegion", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._accessibilityLiveRegion = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "accessibilityComponentType", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._accessibilityComponentType = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "importantForAccessibility", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._importantForAccessibility = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "needsOffscreenAlphaCompositing", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._needsOffscreenAlphaCompositing = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "renderToHardwareTextureAndroid", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._renderToHardwareTextureAndroid = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "nativeBackgroundAndroid", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._nativeBackgroundAndroid = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "accessibilityTraits", {
        /**
         * To be documented
         * @platform ios
         */
        set: function (value) { this._accessibilityTraits = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(HighLevelComponent.prototype, "shouldRasterizeIOS", {
        /**
         * To be documented
         * @platform ios
         */
        set: function (value) { this._shouldRasterizeIOS = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    HighLevelComponent.prototype.setDefaultStyle = function (defaultStyle) {
        this._defaultStyle = defaultStyle;
        this._styleSheet = [defaultStyle];
    };
    HighLevelComponent.prototype.processBoolean = function (value) {
        return value == true || value == 'true';
    };
    HighLevelComponent.prototype.processNumber = function (value) {
        return (!isNaN(parseFloat(value))) ? parseFloat(value) : value;
    };
    HighLevelComponent.prototype.processColor = function (color) {
        return this._wrapper.processColor(color);
    };
    HighLevelComponent.prototype.processEnum = function (value, list) {
        return list.indexOf(value) > -1 ? value : list[0];
    };
    HighLevelComponent.prototype.processDate = function (value) {
        return (new Date(value)).getTime();
    };
    HighLevelComponent.prototype.resolveAssetSource = function (source) {
        return this._wrapper.resolveAssetSource(source);
    };
    HighLevelComponent.prototype.dismissKeyboard = function () {
        this._wrapper.dismissKeyboard();
    };
    HighLevelComponent.prototype.getUIManager = function () {
        return this._wrapper.getUIManager();
    };
    return HighLevelComponent;
}());
export { HighLevelComponent };
//# sourceMappingURL=_component.js.map