var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Component, Inject, Output, EventEmitter } from "@angular/core";
import { REACT_NATIVE_WRAPPER } from "../../renderer/renderer";
import { ReactNativeWrapper } from "../../wrapper/wrapper";
import { HighLevelComponent } from "./_component";
var ANDROID_INPUTS = ['enabled', 'mode', 'prompt'];
var IOS_INPUTS = ['itemStyle'];
var ANDROID_BINDINGS = "[enabled]=\"_enabled\" [mode]=\"_mode\" [prompt]=\"_prompt\" (topSelect)=\"_handleSelect($event)\" [selected]=\"_selectedValue\"";
var IOS_BINDINGS = "[itemStyle]=\"_itemStyle\" onChange=\"true\" (topChange)=\"_handleSelect($event)\" [selectedIndex]=\"_selectedValue\"";
/**
 * A component for displaying a picker.
 *
 * ```
@Component({
  selector: 'sample',
  template: `<Picker [selectedValue]="selected" prompt="Please select an item" [items]="items"
    [style]="{width: 80}" (select)="selected=$event"></Picker>`
})
export class Sample {
  selected: number = 0;
  items: Array<any> = [{label: 'aaa', value: 'a'}, {label: 'bbb', value: 'b'}, {label: 'ccc', value: 'c'},
    {label: 'ddd', value: 'd'}, {label: 'eee', value: 'e'}];
}
 * ```
 * @style https://facebook.github.io/react-native/docs/view.html#style
 */
var Picker = (function (_super) {
    __extends(Picker, _super);
    function Picker(wrapper) {
        var _this = _super.call(this, wrapper) || this;
        //Events
        /**
         * To be documented
         */
        _this.select = new EventEmitter();
        //Properties
        _this._items = [];
        _this._mode = 'dialog';
        _this._itemStyle = false;
        _this.setDefaultStyle({ height: wrapper.isAndroid() ? 50 : 216 });
        return _this;
    }
    Object.defineProperty(Picker.prototype, "selectedValue", {
        /**
         * To be documented
         */
        set: function (value) { this._selectedValue = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "items", {
        /**
         * To be documented
         */
        set: function (value) {
            var _this = this;
            this._items = value.map(function (item) {
                var res = { label: item.label, value: item.value };
                if (item.color) {
                    res['color'] = _this.processColor(item.color);
                }
                return res;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "enabled", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._enabled = this.processBoolean(value); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "mode", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._mode = this.processEnum(value, ['dialog', 'dropdown']); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "prompt", {
        /**
         * To be documented
         * @platform android
         */
        set: function (value) { this._prompt = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Picker.prototype, "itemStyle", {
        /**
         * To be documented
         * @platform ios
         */
        set: function (value) { this._itemStyle = value; },
        enumerable: true,
        configurable: true
    });
    Picker.prototype._handleSelect = function (event) {
        //Event example: {position: 3}
        this._selectedValue = this._wrapper.isAndroid() ? event.position : event.newIndex;
        this.select.emit(this._selectedValue);
    };
    return Picker;
}(HighLevelComponent));
export { Picker };
Picker.decorators = [
    { type: Component, args: [{
                selector: 'Picker',
                inputs: [
                    'items', 'selectedValue',
                    'accessibilityTraits', 'shouldRasterizeIOS',
                    //Both platforms
                    'accessible', 'accessibilityLabel', 'testID', 'pointerEvents', 'removeClippedSubviews', 'onLayout',
                    //Style
                    'styleSheet', 'style', 'itemStyle'
                ],
                template: "\n  <native-dialogpicker *ngIf=\"_mode == 'dialog'\" [items]=\"_items\"\n  [accessibilityTraits]=\"_accessibilityTraits\" [shouldRasterizeIOS]=\"_shouldRasterizeIOS\" [accessible]=\"_accessible\" [accessibilityLabel]=\"_accessibilityLabel\" [testID]=\"_testID\"\n  [pointerEvents]=\"_pointerEvents\" [removeClippedSubviews]=\"_removeClippedSubviews\" [_onLayout]=\"_onLayout\" [styleSheet]=\"_styleSheet\"\n  [style]=\"_style\" [itemStyle]=\"_itemStyle\" onChange=\"true\" (topChange)=\"_handleSelect($event)\" [selectedIndex]=\"_selectedValue\"></native-dialogpicker>\n  <native-dropdownpicker *ngIf=\"_mode == 'dropdown'\" [items]=\"_items\"\n  [accessibilityTraits]=\"_accessibilityTraits\" [shouldRasterizeIOS]=\"_shouldRasterizeIOS\" [accessible]=\"_accessible\" [accessibilityLabel]=\"_accessibilityLabel\" [testID]=\"_testID\"\n  [pointerEvents]=\"_pointerEvents\" [removeClippedSubviews]=\"_removeClippedSubviews\" [_onLayout]=\"_onLayout\" [styleSheet]=\"_styleSheet\"\n  [style]=\"_style\" [itemStyle]=\"_itemStyle\" onChange=\"true\" (topChange)=\"_handleSelect($event)\" [selectedIndex]=\"_selectedValue\"></native-dropdownpicker>"
            },] },
];
/** @nocollapse */
Picker.ctorParameters = function () { return [
    { type: ReactNativeWrapper, decorators: [{ type: Inject, args: [REACT_NATIVE_WRAPPER,] },] },
]; };
Picker.propDecorators = {
    'select': [{ type: Output },],
};
//# sourceMappingURL=_picker.js.map