import { Injector, EventEmitter, NgZone, ElementRef, ViewContainerRef, ComponentFactoryResolver, AfterViewInit, OnDestroy } from "@angular/core";
import { LocationStrategy } from "@angular/common";
import { Router, ActivatedRoute, ChildrenOutletContexts } from "@angular/router";
import { HighLevelComponent } from "./_component";
import { ReactNativeWrapper } from "../../wrapper/wrapper";
import { Node } from "../../renderer/node";
export declare class NavigatorItem extends HighLevelComponent implements AfterViewInit {
    private _componentFactoryResolver;
    private _injector;
    target: ViewContainerRef;
    itemWrapperStyle: any;
    navigationBarHidden: boolean;
    shadowHidden: boolean;
    translucent: boolean;
    barTintColor: number;
    tintColor: number;
    titleTextColor: number;
    _activatedRoute: ActivatedRoute;
    _title: string;
    _titleImage: any;
    _backButtonTitle: string;
    _leftButtonTitle: string;
    _rightButtonTitle: string;
    _backButtonIcon: any;
    _leftButtonIcon: any;
    _rightButtonIcon: any;
    _wrapperStyle: any;
    leftButtonPress: EventEmitter<ActivatedRoute>;
    rightButtonPress: EventEmitter<ActivatedRoute>;
    componentLoad: EventEmitter<any>;
    constructor(_componentFactoryResolver: ComponentFactoryResolver, _injector: Injector, wrapper: ReactNativeWrapper);
    activatedRoute: ActivatedRoute;
    _handleLeftButtonPress(): void;
    _handleRightButtonPress(): void;
    ngAfterViewInit(): void;
}
/**
 * A component for displaying a navigator.
 *
 * It acts as the primary outlet for the router.
 * A Router's navigation triggers a navigation of the Navigator.
 *
 * For each route, additional data can be provided as follows:
 * {title: string, titleImage: any, backButtonIcon: any, backButtonTitle: string, leftButtonIcon: any, leftButtonTitle: string, rightButtonIcon: any, rightButtonTitle: string,
 * wrapperStyle: any, navigationBarHidden: boolean, shadowHidden: boolean, tintColor: string, barTintColor: string, titleTextColor: string, translucent: boolean}
 *
 * ```
@Component({
  selector: 'foo',
  template: `<View routerLink="bar"><Text>Foo from here</Text></View>`
})
class Foo {}

@Component({
  selector: 'bar',
  template: `<View><Text>Bar from here</Text></View>`
})
class Bar {}

@Component({
  selector: 'sample',
  template: `<Navigator (rightButtonPress)="_navigate($event)"></Navigator>`
})
export class Sample {
  constructor(private router: Router) {
  }

  _navigate(event: ActivatedRoute) {
    this.router.navigateByUrl('/bar');
  }
}


const moreLogo = require('../../assets/icon_more.png');
const appRoutes = [
 { path: '', component: Foo, data: {title: 'foo!', rightButtonIcon: moreLogo, backButtonTitle: 'back'}},
 { path: 'bar', component: Bar, data: {title: 'bar!'} }
];

@NgModule({
  declarations: [Sample, CompA, CompB],
  imports: [ReactNativeModule, CommonModule, ReactNativeRouterModule.forRoot(appRoutes)],
  bootstrap: [Sample]
})
export class AppModule {}

 * ```
 * @style https://facebook.github.io/react-native/docs/view.html#style
 * @platform ios
 */
export declare class Navigator extends HighLevelComponent implements OnDestroy {
    private router;
    private zone;
    private locationStrategy;
    private elementRef;
    private parentContexts;
    private _requestedTopOfStack;
    _stack: Array<ActivatedRoute>;
    private _loadedComponents;
    private __wrapper;
    constructor(router: Router, zone: NgZone, locationStrategy: LocationStrategy, elementRef: ElementRef, parentContexts: ChildrenOutletContexts, wrapper: ReactNativeWrapper);
    activateWith(activatedRoute: ActivatedRoute, loadedResolver: ComponentFactoryResolver): void;
    deactivate(): void;
    ngOnDestroy(): void;
    _handleNavigation(lockAcquired: boolean, navigator: Node, activatedRoute: ActivatedRoute): void;
    _handleComponentLoad(cpt: any): void;
    /**
     * To be documented
     */
    readonly activeComponent: any;
    /**
     * To be documented
     */
    leftButtonPress: EventEmitter<ActivatedRoute>;
    /**
     * To be documented
     */
    rightButtonPress: EventEmitter<ActivatedRoute>;
    _barTintColor: number;
    _interactivePopGestureEnabled: boolean;
    _itemWrapperStyle: any;
    _navigationBarHidden: boolean;
    _shadowHidden: boolean;
    _tintColor: number;
    _titleTextColor: number;
    _translucent: boolean;
    /**
     * To be documented
     */
    barTintColor: string;
    /**
     * To be documented
     */
    interactivePopGestureEnabled: any;
    /**
     * To be documented
     */
    itemWrapperStyle: any;
    /**
     * To be documented
     */
    navigationBarHidden: string;
    /**
     * To be documented
     */
    shadowHidden: string;
    /**
     * To be documented
     */
    tintColor: string;
    /**
     * To be documented
     */
    titleTextColor: string;
    /**
     * To be documented
     */
    translucent: string;
    _handleLeftButtonPress(event: ActivatedRoute): void;
    _handleRightButtonPress(event: ActivatedRoute): void;
    _handleNavigationComplete(event: any): void;
}
