import { Headers, BrowserXhr, RequestMethod, Response, ResponseOptions } from "@angular/http";
import { Injectable, NgZone } from "@angular/core";
import { Observable } from "rxjs/Observable";
var ReactNativeXHRConnection = (function () {
    function ReactNativeXHRConnection(req, browserXHR, baseResponseOptions, zone) {
        var _this = this;
        this.zone = zone;
        this.request = req;
        this.response = new Observable(function (responseObserver) {
            var _xhr = browserXHR.build();
            _xhr.open(RequestMethod[req.method].toUpperCase(), req.url);
            // load event handler
            var onLoad = function () {
                // responseText is the old-school way of retrieving response (supported by IE8 & 9)
                // response/responseType properties were introduced in XHR Level2 spec (supported by
                // IE10)
                var body = _xhr.response ? _xhr.response : _xhr.responseText;
                var headers = Headers.fromResponseHeaderString(_xhr.getAllResponseHeaders() || "");
                var url = getResponseURL(_xhr);
                var status = _xhr.status;
                var responseOptions = new ResponseOptions({ body: body, status: status, headers: headers, url: url });
                if (baseResponseOptions) {
                    responseOptions = baseResponseOptions.merge(responseOptions);
                }
                var response = new Response(responseOptions);
                if (isSuccess(status)) {
                    responseObserver.next(response);
                    // TODO(gdi2290): defer complete if array buffer until done
                    responseObserver.complete();
                    return;
                }
                responseObserver.error(response);
            };
            _xhr.onload = function () { return _this.zone.run(function () { return onLoad(); }); };
            var body = _this.request.text();
            if (body && body.length > 0) {
                _xhr.send(body);
            }
            else {
                _xhr.send();
            }
            return function () {
                _xhr.onload = null;
                _xhr.abort();
            };
        });
    }
    return ReactNativeXHRConnection;
}());
var ReactNativeXHRBackend = (function () {
    function ReactNativeXHRBackend(_browserXHR, _baseResponseOptions, _zone) {
        this._browserXHR = _browserXHR;
        this._baseResponseOptions = _baseResponseOptions;
        this._zone = _zone;
    }
    ReactNativeXHRBackend.prototype.createConnection = function (request) {
        return new ReactNativeXHRConnection(request, this._browserXHR, this._baseResponseOptions, this._zone);
    };
    return ReactNativeXHRBackend;
}());
export { ReactNativeXHRBackend };
ReactNativeXHRBackend.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReactNativeXHRBackend.ctorParameters = function () { return [
    { type: BrowserXhr, },
    { type: ResponseOptions, },
    { type: NgZone, },
]; };
var isSuccess = function (status) { return (status >= 200 && status < 300); };
function getResponseURL(xhr) {
    if ('responseURL' in xhr) {
        return xhr.responseURL;
    }
    if (/^X-Request-URL:/m.test(xhr.getAllResponseHeaders())) {
        return xhr.getResponseHeader('X-Request-URL');
    }
    return;
}
//# sourceMappingURL=xhr_backend.js.map