var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { TextNode, nodeMap } from "./node";
var NativeCommand = (function () {
    function NativeCommand(target) {
        this.target = target;
    }
    NativeCommand.prototype.execute = function (wrapper) { };
    NativeCommand.prototype.manageStyleProp = function (wrapper, props) {
        var styles = [];
        styles = styles.concat(this._toArray(props['styleSheet']));
        styles = styles.concat(this._toArray(props['style']));
        if (styles.length > 0) {
            var computedStyle = {};
            try {
                computedStyle = wrapper.computeStyle(styles);
            }
            catch (e) {
                console.error(e);
            }
            for (var key in computedStyle) {
                props[key] = computedStyle[key];
            }
            delete props['styleSheet'];
            delete props['style'];
        }
        return props;
    };
    NativeCommand.prototype._toArray = function (obj) {
        var result = [];
        if (Array.isArray(obj)) {
            result = obj;
        }
        else if (obj) {
            result = [obj];
        }
        return result;
    };
    NativeCommand.addToMergers = function (node, moveTag, addTag, removeTag) {
        var args = { moveTag: moveTag, addTag: addTag, removeTag: removeTag };
        if (!NativeCommand.mergers.has(node)) {
            NativeCommand.mergers.set(node, { initialState: node.nativeChildren.slice(0), commands: [args] });
        }
        else {
            NativeCommand.mergers.get(node).commands.push(args);
        }
    };
    NativeCommand.mergeAndApply = function (wrapper) {
        NativeCommand.mergers.forEach(function (args, node) {
            var initialState = args.initialState;
            var finalState = node.nativeChildren;
            var moveFrom = [], moveTo = [], addTags = [], addAt = [], removeFrom = [];
            args.commands.forEach(function (cmd) {
                if (cmd.moveTag) {
                    moveFrom.push(initialState.indexOf(cmd.moveTag));
                    moveTo.push(finalState.indexOf(cmd.moveTag));
                }
                if (cmd.addTag) {
                    addTags.push(cmd.addTag);
                    addAt.push(finalState.indexOf(cmd.addTag));
                }
                if (cmd.removeTag) {
                    removeFrom.push(initialState.indexOf(cmd.removeTag));
                }
            });
            wrapper.manageChildren(node.nativeTag, moveFrom.length > 0 ? moveFrom : null, moveTo.length > 0 ? moveTo : null, addTags.length > 0 ? addTags : null, addAt.length > 0 ? addAt : null, removeFrom.length > 0 ? removeFrom : null);
            NativeCommand.mergers.delete(node);
        });
    };
    return NativeCommand;
}());
export { NativeCommand };
NativeCommand.mergers = new Map();
var NativeCommandCreate = (function (_super) {
    __extends(NativeCommandCreate, _super);
    function NativeCommandCreate(target) {
        var _this = _super.call(this, target) || this;
        _this.target = target;
        _this.props = {};
        return _this;
    }
    NativeCommandCreate.prototype.execute = function (wrapper) {
        var props = this.target.properties;
        for (var attrName in this.props) {
            props[attrName] = this.props[attrName];
        }
        var ancestor = this.target.getAncestorWithNativeCreated();
        var toBeCreated = false;
        if (!(this.target instanceof TextNode)) {
            toBeCreated = true;
        }
        else {
            toBeCreated = ancestor.isTextContainer();
        }
        if (toBeCreated) {
            var tagName = this.target.overrideTagName(ancestor);
            this.target.nativeTag = wrapper.createView(tagName, 1, this.manageStyleProp(wrapper, props));
            nodeMap.set(this.target.nativeTag, this.target);
        }
        else {
            this.target.isVirtual = true;
            this.target.isCreated = false;
        }
    };
    return NativeCommandCreate;
}(NativeCommand));
export { NativeCommandCreate };
var NativeCommandUpdate = (function (_super) {
    __extends(NativeCommandUpdate, _super);
    function NativeCommandUpdate(target, key, value) {
        var _this = _super.call(this, target) || this;
        _this.target = target;
        _this.key = key;
        _this.value = value;
        _this.props = {};
        _this.props[key] = value;
        return _this;
    }
    NativeCommandUpdate.prototype.execute = function (wrapper) {
        var ancestor = this.target.getAncestorWithNativeCreated();
        var tagName = this.target.overrideTagName(ancestor);
        wrapper.updateView(this.target.nativeTag, tagName, this.manageStyleProp(wrapper, this.props));
    };
    return NativeCommandUpdate;
}(NativeCommand));
export { NativeCommandUpdate };
var NativeCommandAttach = (function (_super) {
    __extends(NativeCommandAttach, _super);
    function NativeCommandAttach(target, toRoot) {
        var _this = _super.call(this, target) || this;
        _this.target = target;
        _this.toRoot = toRoot;
        return _this;
    }
    NativeCommandAttach.prototype.execute = function (wrapper) {
        if (this.toRoot) {
            wrapper.manageChildren(1, null, null, [this.target.nativeTag], [0], null);
        }
        else {
            if (this.target.parent) {
                var ancestor = this.target.getAncestorWithNativeCreated();
                if (ancestor) {
                    if (this.target.isCreated && !this.target.isVirtual) {
                        NativeCommand.addToMergers(ancestor, null, this.target.nativeTag, null);
                        ancestor.nativeChildren.push(this.target.nativeTag);
                    }
                    else {
                        this.target.children.forEach(function (child) {
                            if (child.isCreated && !child.isVirtual) {
                                NativeCommand.addToMergers(ancestor, null, child.nativeTag, null);
                                ancestor.nativeChildren.push(child.nativeTag);
                            }
                        });
                    }
                }
            }
        }
    };
    return NativeCommandAttach;
}(NativeCommand));
export { NativeCommandAttach };
var NativeCommandAttachAfter = (function (_super) {
    __extends(NativeCommandAttachAfter, _super);
    function NativeCommandAttachAfter(target, anchor) {
        var _this = _super.call(this, target) || this;
        _this.target = target;
        _this.anchor = anchor;
        return _this;
    }
    NativeCommandAttachAfter.prototype.executeWithCounters = function (wrapper, counters) {
        var _this = this;
        if (this.target.parent) {
            var ancestor = this.target.getAncestorWithNativeCreated();
            if (ancestor) {
                var baseNativeIndex = this.anchor.getInsertionNativeIndex();
                if (this.target.isCreated && !this.target.isVirtual) {
                    this._attach(wrapper, this.target, ancestor, counters, baseNativeIndex);
                }
                else {
                    this.target.children.forEach(function (child) {
                        if (child.isCreated && !child.isVirtual) {
                            _this._attach(wrapper, child, ancestor, counters, baseNativeIndex);
                        }
                    });
                }
            }
        }
    };
    NativeCommandAttachAfter.prototype._attach = function (wrapper, node, ancestor, counters, baseNativeIndex) {
        var shift = counters.get(this.anchor) || 0;
        var nativeIndex = baseNativeIndex + shift;
        if (!node.toBeMoved) {
            NativeCommand.addToMergers(ancestor, null, node.nativeTag, null);
            ancestor.nativeChildren.splice(nativeIndex, 0, node.nativeTag);
        }
        else {
            var currentIndex = ancestor.nativeChildren.indexOf(node.nativeTag);
            if (currentIndex != nativeIndex) {
                NativeCommand.addToMergers(ancestor, node.nativeTag, null, null);
                ancestor.nativeChildren.splice(currentIndex, 1);
                ancestor.nativeChildren.splice(nativeIndex, 0, node.nativeTag);
            }
            node.toBeMoved = false;
        }
        counters.set(this.anchor, shift + 1);
    };
    return NativeCommandAttachAfter;
}(NativeCommand));
export { NativeCommandAttachAfter };
var NativeCommandDetach = (function (_super) {
    __extends(NativeCommandDetach, _super);
    function NativeCommandDetach(target) {
        var _this = _super.call(this, target) || this;
        _this.target = target;
        return _this;
    }
    NativeCommandDetach.prototype.execute = function (wrapper) {
        var parent = this.target.parent;
        var toDetach = this.target.nativeTag > -1 ? this.target : (this.target.isVirtual ? this.target.getFirstCreatedChild() : null);
        if (toDetach) {
            if (toDetach.toBeDestroyed) {
                var nativeIndex = parent.nativeChildren.indexOf(toDetach.nativeTag);
                if (toDetach.getAncestorDestroyed() == null) {
                    NativeCommand.addToMergers(parent, null, null, toDetach.nativeTag);
                }
                parent.nativeChildren.splice(nativeIndex, 1);
                toDetach.destroyNative();
            }
            else {
                //Node is being moved
                this.target.toBeMoved = true;
            }
        }
    };
    return NativeCommandDetach;
}(NativeCommand));
export { NativeCommandDetach };
//# sourceMappingURL=native_command.js.map