var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Hammer } from "./../events/hammer";
import { SelectorMatcher, CssSelector } from "./selector";
export var nodeMap = new Map();
var Node = (function () {
    function Node(rnWrapper, zoneHolder) {
        this.rnWrapper = rnWrapper;
        this.zoneHolder = zoneHolder;
        this.children = [];
        this.tagName = "";
        this.properties = {};
        this.isVirtual = false;
        this.toBeDestroyed = false;
        this.toBeMoved = false;
        this.isDestroyed = false;
        this.nativeTag = -1;
        this.isCreated = false;
        this.nativeChildren = [];
        this.eventListeners = new Map();
        this._hammer = null;
    }
    Node.prototype.attachTo = function (parent) {
        if (parent) {
            parent.children.push(this);
            this.parent = parent;
        }
    };
    Node.prototype.attachToAt = function (parent, index) {
        if (parent) {
            parent.children.splice(index, 0, this);
            this.parent = parent;
        }
    };
    Node.prototype.getInsertionNativeIndex = function () {
        var nativeIndex = -1;
        var ancestor = this.getAncestorWithNativeCreated();
        if (ancestor) {
            var childrenOfAncestor = this;
            while (childrenOfAncestor.parent != ancestor) {
                childrenOfAncestor = childrenOfAncestor.parent;
            }
            var index = ancestor.children.indexOf(childrenOfAncestor);
            var count = index;
            while (count >= 0) {
                var prev = ancestor.children[count];
                if (prev.nativeTag > -1) {
                    nativeIndex = ancestor.nativeChildren.indexOf(prev.nativeTag);
                    count = 0;
                }
                else if (prev.isVirtual) {
                    var nativeChild = prev.getFirstCreatedChild();
                    if (nativeChild) {
                        nativeIndex = ancestor.nativeChildren.indexOf(nativeChild.nativeTag);
                        if (nativeIndex > -1) {
                            count = 0;
                        }
                    }
                }
                count--;
            }
        }
        return nativeIndex + 1;
    };
    Node.prototype.getAncestorWithNativeCreated = function () {
        var next = this.parent;
        while (next && !next.isCreated) {
            next = next.parent;
        }
        return next;
    };
    Node.prototype.getAncestorDestroyed = function () {
        var next = this.parent;
        while (next && !next.isDestroyed) {
            next = next.parent;
        }
        return next;
    };
    Node.prototype.getFirstCreatedChild = function () {
        var result = null;
        for (var i = 0; i < this.children.length; i++) {
            var child = this.children[i];
            if (child.isCreated) {
                result = child;
                break;
            }
        }
        return result;
    };
    Node.prototype.isTextContainer = function () {
        return this.tagName == 'native-text' || this.tagName == 'VirtualText';
    };
    Node.prototype.isImageContainer = function () {
        return this.tagName == 'native-image';
    };
    Node.prototype.overrideTagName = function (ancestor) {
        var tagName = this.tagName;
        if (this.isTextContainer() && ancestor.isTextContainer()) {
            tagName = 'native-virtualtext';
        }
        else if (this.isImageContainer() && ancestor.isTextContainer()) {
            tagName = 'native-inlineimage';
        }
        else if (tagName == 'native-scrollview' && this.properties['horizontal'] && this.rnWrapper.isAndroid()) {
            tagName = 'native-horizontalscrollview';
        }
        else if (tagName == 'native-textinput' && this.properties['multiline'] && !this.rnWrapper.isAndroid()) {
            tagName = 'native-textarea';
        }
        return tagName;
    };
    Node.prototype.destroyNative = function () {
        this.isDestroyed = true;
        this.isCreated = false;
        nodeMap.delete(this.nativeTag);
        this.nativeTag = -1;
        this.nativeChildren = [];
        this.toBeDestroyed = this.toBeMoved = false;
        for (var i = 0; i < this.children.length; i++) {
            this.children[i].destroyNative();
        }
    };
    Node.prototype.setProperty = function (name, value, updateNative) {
        if (updateNative === void 0) { updateNative = true; }
        this.properties[name] = value;
        if (updateNative && this.nativeTag > -1) {
            var prop = {};
            prop[name] = value;
            this.rnWrapper.updateView(this.nativeTag, this.tagName, prop);
        }
    };
    Node.prototype.setProperties = function (properties) {
        for (var propName in properties) {
            this.properties[propName] = properties[propName];
        }
        this.rnWrapper.updateView(this.nativeTag, this.tagName, properties);
    };
    Node.prototype.addEventListener = function (eventName, handler) {
        if (!Hammer.supports(eventName)) {
            if (!this.eventListeners.has(eventName)) {
                this.eventListeners.set(eventName, []);
            }
            var handlers = this.eventListeners.get(eventName);
            handlers.push(handler);
            this.eventListeners.set(eventName, handlers);
        }
        else {
            if (this._hammer == null) {
                this._hammer = Hammer.create(this);
            }
            Hammer.add(this._hammer, eventName, handler);
        }
    };
    Node.prototype.removeEventListener = function (eventName, handler) {
        if (!Hammer.supports(eventName)) {
            var handlers = this.eventListeners.get(eventName);
            if (handlers) {
                var index = handlers.indexOf(handler);
                if (index > -1) {
                    handlers.splice(index, 1);
                    this.eventListeners.set(eventName, handlers);
                }
            }
        }
        else {
            this._hammer = Hammer.remove(this._hammer, eventName, handler);
        }
    };
    Node.prototype.fireEvent = function (name, event) {
        var _this = this;
        event.currentTarget = this;
        var handlers = this.eventListeners.get(name);
        if (handlers) {
            handlers.forEach(function (handler) {
                _this.zoneHolder.zone.run(function () { return handler(event); });
            });
        }
        if (this.parent && !event._stop) {
            this.parent.fireEvent(name, event);
        }
    };
    //TODO: generalize this TextInput specific code
    Node.prototype.focus = function () {
        if (this.rnWrapper.isAndroid()) {
            this.rnWrapper.dispatchCommand(this.nativeTag, 'focusTextInput', null);
        }
        else {
            this.rnWrapper.focus(this.nativeTag);
        }
    };
    Node.prototype.blur = function () {
        if (this.rnWrapper.isAndroid()) {
            this.rnWrapper.dispatchCommand(this.nativeTag, 'blurTextInput', null);
        }
        else {
            this.rnWrapper.blur(this.nativeTag);
        }
    };
    Node.prototype.dispatchCommand = function (command, params) {
        if (params === void 0) { params = null; }
        if (this.nativeTag !== -1) {
            this.rnWrapper.dispatchCommand(this.nativeTag, command, params);
        }
    };
    Node.prototype.getElementByTestId = function (testId) {
        return this.querySelector('#' + testId);
    };
    Node.prototype.querySelectorAll = function (selector) {
        var _this = this;
        var res = [];
        var _recursive = function (result, node, selector, matcher) {
            var cNodes = node.children;
            if (cNodes && cNodes.length > 0) {
                for (var i = 0; i < cNodes.length; i++) {
                    var childNode = cNodes[i];
                    if (childNode.isVirtual && _this._elementMatches(childNode, selector, matcher)) {
                        result.push(childNode);
                    }
                    _recursive(result, childNode, selector, matcher);
                }
            }
        };
        var matcher = new SelectorMatcher();
        matcher.addSelectables(CssSelector.parse(selector));
        _recursive(res, this, selector, matcher);
        return res;
    };
    Node.prototype.querySelector = function (selector) {
        return this.querySelectorAll(selector)[0];
    };
    Node.prototype._elementMatches = function (node, selector, matcher) {
        if (matcher === void 0) { matcher = null; }
        if (selector === '*') {
            return true;
        }
        var result = false;
        if (selector && selector.charAt(0) == "#") {
            result = node.properties['testID'] == selector.substring(1);
        }
        else if (selector) {
            var result = false;
            if (matcher == null) {
                matcher = new SelectorMatcher();
                matcher.addSelectables(CssSelector.parse(selector));
            }
            var cssSelector = new CssSelector();
            cssSelector.setElement(node.tagName);
            if (node.properties) {
                for (var propName in node.properties) {
                    cssSelector.addAttribute(propName, node.properties[propName]);
                }
            }
            matcher.match(cssSelector, function (selector, cb) { result = true; });
        }
        return result;
    };
    return Node;
}());
export { Node };
var ElementNode = (function (_super) {
    __extends(ElementNode, _super);
    function ElementNode(tagName, wrapper, zoneHolder) {
        var _this = _super.call(this, wrapper, zoneHolder) || this;
        _this.tagName = tagName;
        //TODO: generalize the mechanism (list? regexp? meta data?)
        if (['dummy-anchor-for-dynamic-loader', 'View', 'Text', 'Switch', 'TextInput', 'WebView', 'Image', 'ProgressBar', 'PagerLayout', 'Picker', 'ScrollView',
            'DrawerLayout', 'DrawerLayoutSide', 'DrawerLayoutContent', 'RefreshControl', 'Toolbar',
            'ActivityIndicator', 'DatePicker', 'Navigator', 'NavigatorItem', 'ProgressView', 'SegmentedControl', 'Slider', 'TabBar', 'TabBarItem'].indexOf(tagName) > -1) {
            _this.isVirtual = true;
        }
        return _this;
    }
    return ElementNode;
}(Node));
export { ElementNode };
var TextNode = (function (_super) {
    __extends(TextNode, _super);
    function TextNode(value, wrapper, zoneHolder) {
        var _this = _super.call(this, wrapper, zoneHolder) || this;
        _this.tagName = 'native-rawtext';
        _this.setText(value);
        return _this;
    }
    TextNode.prototype.setText = function (text) {
        var trimmedText = text ? text.trim() : '';
        this.setProperty('text', trimmedText, false);
        return trimmedText;
    };
    return TextNode;
}(Node));
export { TextNode };
var CommentNode = (function (_super) {
    __extends(CommentNode, _super);
    function CommentNode(wrapper, zoneHolder) {
        var _this = _super.call(this, wrapper, zoneHolder) || this;
        _this.isVirtual = true;
        return _this;
    }
    return CommentNode;
}(Node));
export { CommentNode };
//# sourceMappingURL=node.js.map