import { RendererFactory2, Renderer2, RendererType2, RendererStyleFlags2, InjectionToken, NgZone, Sanitizer, SecurityContext, SchemaMetadata } from "@angular/core";
import { ElementSchemaRegistry } from "@angular/compiler";
import { Node } from "./node";
import { ReactNativeWrapper } from "./../wrapper/wrapper";
export declare const REACT_NATIVE_WRAPPER: InjectionToken<string>;
export declare class ReactNativeElementSchemaRegistry extends ElementSchemaRegistry {
    getDefaultComponentElementName(): string;
    hasProperty(tagName: string, propName: string): boolean;
    hasElement(tagName: string, schemaMetas: SchemaMetadata[]): boolean;
    getMappedPropName(propName: string): string;
    securityContext(tagName: string, propName: string): any;
    validateProperty(name: string): {
        error: boolean;
        msg?: string;
    };
    validateAttribute(name: string): {
        error: boolean;
        msg?: string;
    };
    allKnownElementNames(): string[];
    normalizeAnimationStyleProperty(propName: string): string;
    normalizeAnimationStyleValue(camelCaseProp: string, userProvidedProp: string, val: string | number): {
        error: string;
        value: string;
    };
}
export declare class ReactNativeSanitizer implements Sanitizer {
    sanitize(ctx: SecurityContext, value: any): string;
}
export declare class ReactNativeRootRenderer implements RendererFactory2 {
    zone: NgZone;
    wrapper: ReactNativeWrapper;
    private _registeredComponents;
    private _createCommands;
    private _updateCommands;
    private _attachCommands;
    private _attachAfterCommands;
    private _detachCommands;
    constructor(_wrapper: ReactNativeWrapper);
    createRenderer(hostElement: any, type: RendererType2 | any): Renderer2;
    addCreateCommand(node: Node, props?: {
        [s: string]: any;
    }): void;
    addUpdateCommand(node: Node, key: string, value: any): void;
    addAttachCommand(node: Node, toRoot: boolean): void;
    addAttachAfterCommand(node: Node, anchor: Node): void;
    addDetachCommand(node: Node): void;
    executeCommands(): void;
}
export declare class ReactNativeRenderer implements Renderer2 {
    private _rootRenderer;
    data: {
        [key: string]: any;
    };
    constructor(_rootRenderer: ReactNativeRootRenderer);
    selectRootElement(selector: string): Node;
    createElement(name: string, namespace?: string | any): Node;
    createComment(value: string): any;
    createText(value: string): Node;
    destroyNode(node: Node): any;
    destroy(): void;
    appendChild(parent: Node, newChild: Node): void;
    insertBefore(parent: any, newChild: any, refChild: any): void;
    removeChild(parent: any, oldChild: any): void;
    parentNode(node: Node): Node;
    nextSibling(node: Node): any;
    setAttribute(el: Node, name: string, value: string, namespace?: string | any): void;
    removeAttribute(el: any, name: string, namespace?: string | any): void;
    addClass(el: any, name: string): void;
    removeClass(el: any, name: string): void;
    setStyle(el: any, style: string, value: any, flags?: RendererStyleFlags2): void;
    removeStyle(el: any, style: string, flags?: RendererStyleFlags2): void;
    setProperty(el: Node, name: string, value: any): void;
    setValue(node: Node, value: string): void;
    listen(target: 'window' | 'document' | 'body' | Node, eventName: string, callback: (event: any) => boolean | void): () => void;
    private _createElementCommand(node);
    private _createTextCommand(node);
    private _createNativeRecursively(node, isRoot?);
}
