var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { InjectionToken, Inject, Injectable } from "@angular/core";
import { ElementSchemaRegistry } from "@angular/compiler";
import { ElementNode, TextNode, nodeMap, CommentNode } from "./node";
import { ReactNativeWrapper } from "./../wrapper/wrapper";
import { NativeCommand, NativeCommandCreate, NativeCommandUpdate, NativeCommandAttach, NativeCommandDetach, NativeCommandAttachAfter } from "./native_command";
export var REACT_NATIVE_WRAPPER = new InjectionToken("ReactNativeWrapper");
var ReactNativeElementSchemaRegistry = (function (_super) {
    __extends(ReactNativeElementSchemaRegistry, _super);
    function ReactNativeElementSchemaRegistry() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ReactNativeElementSchemaRegistry.prototype.getDefaultComponentElementName = function () {
        return 'def-cpt';
    };
    ReactNativeElementSchemaRegistry.prototype.hasProperty = function (tagName, propName) {
        return true;
    };
    ReactNativeElementSchemaRegistry.prototype.hasElement = function (tagName, schemaMetas) {
        return true;
    };
    ReactNativeElementSchemaRegistry.prototype.getMappedPropName = function (propName) {
        return propName;
    };
    ReactNativeElementSchemaRegistry.prototype.securityContext = function (tagName, propName) {
        return 0;
    };
    ReactNativeElementSchemaRegistry.prototype.validateProperty = function (name) {
        return { error: false };
    };
    ReactNativeElementSchemaRegistry.prototype.validateAttribute = function (name) {
        return { error: false };
    };
    ReactNativeElementSchemaRegistry.prototype.allKnownElementNames = function () {
        return [];
    };
    ReactNativeElementSchemaRegistry.prototype.normalizeAnimationStyleProperty = function (propName) {
        return propName;
    };
    ReactNativeElementSchemaRegistry.prototype.normalizeAnimationStyleValue = function (camelCaseProp, userProvidedProp, val) {
        return { error: null, value: '' + val };
    };
    return ReactNativeElementSchemaRegistry;
}(ElementSchemaRegistry));
export { ReactNativeElementSchemaRegistry };
var ReactNativeSanitizer = (function () {
    function ReactNativeSanitizer() {
    }
    ReactNativeSanitizer.prototype.sanitize = function (ctx, value) {
        return value;
    };
    return ReactNativeSanitizer;
}());
export { ReactNativeSanitizer };
var ReactNativeRootRenderer = (function () {
    function ReactNativeRootRenderer(_wrapper) {
        this._registeredComponents = new Map();
        this._createCommands = new Map();
        this._updateCommands = new Map();
        this._attachCommands = new Map();
        this._attachAfterCommands = new Map();
        this._detachCommands = new Map();
        this.wrapper = _wrapper;
        this.wrapper.patchReactNativeEventEmitter(nodeMap);
    }
    ReactNativeRootRenderer.prototype.createRenderer = function (hostElement, type) {
        return new ReactNativeRenderer(this);
    };
    ReactNativeRootRenderer.prototype.addCreateCommand = function (node, props) {
        if (props === void 0) { props = null; }
        var cmd = new NativeCommandCreate(node);
        if (props) {
            cmd.props = props;
        }
        this._createCommands.set(node, cmd);
    };
    ReactNativeRootRenderer.prototype.addUpdateCommand = function (node, key, value) {
        var propEater = this._createCommands.get(node) || this._updateCommands.get(node);
        if (propEater) {
            propEater.props[key] = value;
        }
        else {
            this._updateCommands.set(node, new NativeCommandUpdate(node, key, value));
        }
    };
    ReactNativeRootRenderer.prototype.addAttachCommand = function (node, toRoot) {
        this._attachCommands.set(node, new NativeCommandAttach(node, toRoot));
    };
    ReactNativeRootRenderer.prototype.addAttachAfterCommand = function (node, anchor) {
        this._attachAfterCommands.set(node, new NativeCommandAttachAfter(node, anchor));
    };
    ReactNativeRootRenderer.prototype.addDetachCommand = function (node) {
        this._detachCommands.set(node, new NativeCommandDetach(node));
    };
    ReactNativeRootRenderer.prototype.executeCommands = function () {
        var _this = this;
        this._detachCommands.forEach(function (command) { return command.execute(_this.wrapper); });
        this._createCommands.forEach(function (command) { return command.execute(_this.wrapper); });
        this._updateCommands.forEach(function (command) { return command.execute(_this.wrapper); });
        this._attachCommands.forEach(function (command) { return command.execute(_this.wrapper); });
        var counters = new Map();
        this._attachAfterCommands.forEach(function (command) { return command.executeWithCounters(_this.wrapper, counters); });
        counters.clear();
        NativeCommand.mergeAndApply(this.wrapper);
        this._detachCommands.clear();
        this._createCommands.clear();
        this._updateCommands.clear();
        this._attachCommands.clear();
        this._attachAfterCommands.clear();
    };
    return ReactNativeRootRenderer;
}());
export { ReactNativeRootRenderer };
ReactNativeRootRenderer.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReactNativeRootRenderer.ctorParameters = function () { return [
    { type: ReactNativeWrapper, decorators: [{ type: Inject, args: [REACT_NATIVE_WRAPPER,] },] },
]; };
var ReactNativeRenderer = (function () {
    function ReactNativeRenderer(_rootRenderer) {
        this._rootRenderer = _rootRenderer;
        this.data = {};
    }
    ReactNativeRenderer.prototype.selectRootElement = function (selector) {
        var root = this.createElement(selector.startsWith('#root') ? 'test-cmp' : selector);
        this._createElementCommand(root);
        this._rootRenderer.addAttachCommand(root, true);
        return root;
    };
    ReactNativeRenderer.prototype.createElement = function (name, namespace) {
        //console.log('createElement:' + name);
        return new ElementNode(name, this._rootRenderer.wrapper, this._rootRenderer);
    };
    ReactNativeRenderer.prototype.createComment = function (value) {
        return new CommentNode(this._rootRenderer.wrapper, this._rootRenderer);
    };
    ReactNativeRenderer.prototype.createText = function (value) {
        //console.log('createText:' + value);
        return new TextNode(value, this._rootRenderer.wrapper, this._rootRenderer);
    };
    ReactNativeRenderer.prototype.destroyNode = function (node) {
        node.toBeDestroyed = true;
    };
    ReactNativeRenderer.prototype.destroy = function () {
        //console.log('NOT IMPLEMENTED: destroy', arguments);
    };
    ReactNativeRenderer.prototype.appendChild = function (parent, newChild) {
        //console.log('appendChild');
        newChild.attachTo(parent);
        if (newChild.getAncestorWithNativeCreated()) {
            if (this._createNativeRecursively(newChild)) {
                this._rootRenderer.addAttachCommand(newChild, false);
            }
        }
    };
    ReactNativeRenderer.prototype.insertBefore = function (parent, newChild, refChild) {
        //console.log('insertBefore');
        var index = parent.children.indexOf(refChild);
        newChild.attachToAt(parent, index);
        if (newChild.getAncestorWithNativeCreated()) {
            if (this._createNativeRecursively(newChild)) {
                this._rootRenderer.addAttachCommand(newChild, false);
            }
        }
    };
    ReactNativeRenderer.prototype.removeChild = function (parent, oldChild) {
        var index = parent.children.indexOf(oldChild);
        parent.children.splice(index, 1);
        this._rootRenderer.addDetachCommand(oldChild);
    };
    ReactNativeRenderer.prototype.parentNode = function (node) {
        return node.parent;
    };
    ReactNativeRenderer.prototype.nextSibling = function (node) {
        var res = null;
        var parent = node.parent;
        if (parent) {
            var index = parent.children.indexOf(node) + 1;
            if (parent.children.length > index) {
                res = parent.children[index];
            }
        }
        return res;
    };
    ReactNativeRenderer.prototype.setAttribute = function (el, name, value, namespace) {
        var val = value;
        if (name == "ng-version")
            return;
        if (value == "false")
            val = false;
        if (value == "true")
            val = true;
        if (value == "null")
            val = null;
        if (!isNaN(parseInt(val)))
            val = parseInt(val);
        if (value.startsWith('#'))
            val = this._rootRenderer.wrapper.processColor(value);
        this.setProperty(el, name, val);
    };
    ReactNativeRenderer.prototype.removeAttribute = function (el, name, namespace) {
    };
    ReactNativeRenderer.prototype.addClass = function (el, name) {
        console.error('NOT IMPLEMENTED: addClass', arguments);
    };
    ReactNativeRenderer.prototype.removeClass = function (el, name) {
        console.error('NOT IMPLEMENTED: removeClass', arguments);
    };
    ReactNativeRenderer.prototype.setStyle = function (el, style, value, flags) {
        console.error('NOT IMPLEMENTED: setStyle', arguments);
    };
    ReactNativeRenderer.prototype.removeStyle = function (el, style, flags) {
        console.error('NOT IMPLEMENTED: removeStyle', arguments);
    };
    ReactNativeRenderer.prototype.setProperty = function (el, name, value) {
        if (typeof value !== 'undefined') {
            var cleanPropertyName = name.startsWith('_on') ? name.substr(1) : name;
            el.setProperty(cleanPropertyName, value, false);
            if (el.isCreated) {
                this._rootRenderer.addUpdateCommand(el, cleanPropertyName, value);
            }
        }
    };
    ReactNativeRenderer.prototype.setValue = function (node, value) {
        if (node instanceof TextNode) {
            var trimedText = node.setText(value);
            this.setProperty(node, 'text', trimedText);
        }
    };
    ReactNativeRenderer.prototype.listen = function (target, eventName, callback) {
        if (target === 'window' || target == 'document' || target == 'body') {
            console.error('NOT IMPLEMENTED: listen on ' + target, arguments);
            return function () { };
        }
        else {
            target.addEventListener(eventName, callback);
            return function () { target.removeEventListener(eventName, callback); };
        }
    };
    ReactNativeRenderer.prototype._createElementCommand = function (node) {
        this._rootRenderer.addCreateCommand(node);
        node.isCreated = true;
    };
    ReactNativeRenderer.prototype._createTextCommand = function (node) {
        this._rootRenderer.addCreateCommand(node, { text: node.properties['text'] });
        var cmd = new NativeCommandCreate(node);
        node.isCreated = true;
    };
    ReactNativeRenderer.prototype._createNativeRecursively = function (node, isRoot) {
        if (isRoot === void 0) { isRoot = true; }
        var didCreate = false;
        if (!node.isCreated) {
            if (!node.isVirtual) {
                node instanceof TextNode ? this._createTextCommand(node) : this._createElementCommand(node);
                didCreate = !(node instanceof TextNode) || isRoot;
            }
            for (var i = 0; i < node.children.length; i++) {
                var child = node.children[i];
                didCreate = this._createNativeRecursively(child, false) || didCreate;
                if (!child.isVirtual && !(isRoot && node.isVirtual)) {
                    this._rootRenderer.addAttachCommand(child, false);
                }
            }
        }
        return didCreate;
    };
    return ReactNativeRenderer;
}());
export { ReactNativeRenderer };
//# sourceMappingURL=renderer.js.map