"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var wrapper_1 = require("./wrapper");
var Command = (function () {
    function Command(name, target, details) {
        this.name = name;
        this.target = target;
        this.details = details;
    }
    Command.prototype.toString = function () {
        return this.name + "+" + this.target + "+" + this.details;
    };
    return Command;
}());
exports.Command = Command;
var MockReactNativeWrapper = (function (_super) {
    __extends(MockReactNativeWrapper, _super);
    function MockReactNativeWrapper() {
        _super.call(this);
        this.reset();
    }
    MockReactNativeWrapper.prototype.reset = function () {
        this.commandLogs = [];
        this.root = new NativeElement('root', 1, {});
        this.nativeElementMap = new Map();
        this._lastTagUsed = 1;
        this.nativeElementMap.set(1, this.root);
        this._platform = 'android';
    };
    MockReactNativeWrapper.prototype.clearLogs = function () {
        this.commandLogs = [];
    };
    MockReactNativeWrapper.prototype.createView = function (tagName, root, properties) {
        this._lastTagUsed++;
        var element = new NativeElement(tagName, this._lastTagUsed, properties);
        this.nativeElementMap.set(this._lastTagUsed, element);
        this.commandLogs.push(new Command('CREATE', this._lastTagUsed, tagName + '+' + JSON.stringify(properties)));
        return this._lastTagUsed;
    };
    MockReactNativeWrapper.prototype.updateView = function (tag, tagName, properties) {
        var element = this.nativeElementMap.get(tag);
        element.name = tagName;
        for (var key in properties) {
            element.properties[key] = properties[key];
        }
        this.commandLogs.push(new Command('UPDATE', tag, tagName + '+' + JSON.stringify(properties)));
    };
    // moveFrom and removeFrom are both relative to the starting state of the View's children.
    // moveTo and addAt are both relative to the final state of the View's children.
    MockReactNativeWrapper.prototype.manageChildren = function (parentTag, moveFrom, moveTo, addTags, addAt, removeFrom) {
        var parentElement = this.nativeElementMap.get(parentTag);
        var toBeDeleted = [];
        var toBeAdded = [];
        if (moveFrom && moveTo && moveFrom.length != moveTo.length) {
            throw new Error("manageChildren - MOVE - Invalid lengths: " + moveFrom.length + " vs " + moveTo.length);
        }
        if (addTags && addAt && addTags.length != addAt.length) {
            throw new Error("manageChildren - MOVE - Invalid lengths: " + addTags.length + " vs " + addAt.length);
        }
        //Detach commands
        if (removeFrom) {
            for (var i = 0; i < removeFrom.length; i++) {
                var index = removeFrom[removeFrom.length - i - 1];
                toBeDeleted.push(index);
                this.commandLogs.push(new Command('DETACH', parentTag, '' + index));
            }
        }
        //Move commands
        if (moveFrom) {
            for (var i = 0; i < moveFrom.length; i++) {
                var tag = parentElement.children[moveFrom[i]].tag;
                toBeDeleted.push(moveFrom[i]);
                toBeAdded.push({ index: moveTo[i], tag: tag });
            }
            this.commandLogs.push(new Command('MOVE', parentTag, '' + moveFrom.join(',') + '+' + moveTo.join(',')));
        }
        //Attach commands
        if (addTags) {
            for (var i = 0; i < addTags.length; i++) {
                toBeAdded.push({ index: addAt[i], tag: addTags[i] });
                this.commandLogs.push(new Command('ATTACH', parentTag, addTags[i] + '+' + addAt[i]));
            }
        }
        //Update data structure
        toBeDeleted.sort();
        toBeAdded.sort(function (a, b) { return a.index - b.index; });
        for (var i = 0; i < toBeDeleted.length; i++) {
            parentElement.children.splice(toBeDeleted[toBeDeleted.length - i - 1], 1);
        }
        for (var i = 0; i < toBeAdded.length; i++) {
            var item = toBeAdded[i];
            var element = this.nativeElementMap.get(item.tag);
            element.parent = parentElement;
            if (item.index <= parentElement.children.length) {
                parentElement.children.splice(item.index, 0, element);
            }
            else {
                throw new Error("manageChildren - ATTACH - Invalid index " + item.index + ",  size is " + parentElement.children.length);
            }
        }
    };
    MockReactNativeWrapper.prototype.dispatchCommand = function (tag, command, params) {
        if (params === void 0) { params = null; }
        this.commandLogs.push(new Command('COMMAND', tag, command + (params ? '+' + params.toString() : "")));
    };
    MockReactNativeWrapper.prototype.computeStyle = function (styles) {
        var res = {};
        styles.forEach(function (style) {
            if (!isNaN(parseInt(style))) {
                res['flex'] = 1;
                res['collapse'] = true;
            }
            else {
                for (var key in style) {
                    res[key] = style[key];
                }
            }
        });
        return res;
    };
    MockReactNativeWrapper.prototype.patchReactNativeWithZone = function (zone) {
        //Not needed in Mock
    };
    MockReactNativeWrapper.prototype.patchReactNativeEventEmitter = function (nodeMap) {
        //Not needed in Mock
    };
    MockReactNativeWrapper.prototype.processColor = function (color) {
        return 42;
    };
    MockReactNativeWrapper.prototype.resolveAssetSource = function (source) {
        return source;
    };
    MockReactNativeWrapper.prototype.dismissKeyboard = function () {
        this.commandLogs.push(new Command('DISMISS_KEYBOARD', -1, ''));
    };
    MockReactNativeWrapper.prototype.requestNavigatorLock = function (tag, callback) {
        this.commandLogs.push(new Command('REQUEST_NAVIGATOR_LOCK', tag, ''));
        callback(true);
    };
    MockReactNativeWrapper.prototype.getUIManager = function () {
        return {
            AndroidDrawerLayout: {
                Constants: { DrawerPosition: { Right: 1, Left: -1 } }
            },
            AndroidSwipeRefreshLayout: {
                Constants: { SIZE: { DEFAULT: 1, LARGE: 0 } }
            },
            ToolbarAndroid: {
                Constants: { ShowAsAction: { always: 2, never: 0, ifRoom: 1 } }
            },
            RCTScrollView: {
                Constants: { DecelerationRate: { normal: 0.9, fast: 0.998 } }
            }
        };
    };
    MockReactNativeWrapper.prototype.isAndroid = function () {
        return this._platform == 'android';
    };
    MockReactNativeWrapper.prototype.blur = function (tag) {
        this.commandLogs.push(new Command('COMMAND', tag, 'blur'));
    };
    MockReactNativeWrapper.prototype.focus = function (tag) {
        this.commandLogs.push(new Command('COMMAND', tag, 'focus'));
    };
    MockReactNativeWrapper.prototype.setPlatform = function (platform) {
        this._platform = platform;
        wrapper_1.overridePlatform(platform);
    };
    MockReactNativeWrapper.prototype.$log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        //console.log(...args);
    };
    return MockReactNativeWrapper;
}(wrapper_1.ReactNativeWrapper));
exports.MockReactNativeWrapper = MockReactNativeWrapper;
var NativeElement = (function () {
    function NativeElement(name, tag, properties) {
        this.name = name;
        this.tag = tag;
        this.properties = properties;
        this.parent = null;
        this.children = [];
    }
    NativeElement.prototype.toString = function (shift) {
        if (shift === void 0) { shift = 0; }
        var res = '';
        for (var i = 0; i < shift; i++)
            res += ' ';
        res += this.name + ':' + JSON.stringify(this.properties) + '\n';
        this.children.forEach(function (child) {
            res += child.toString(shift + 2);
        });
        if (shift) {
            return res;
        }
        else {
            return res.trim();
        }
    };
    return NativeElement;
}());
exports.NativeElement = NativeElement;
